@@ -56,7 +56,7 @@ require Perl::Dist::WiX::Asset::Perl;
 require Perl::Dist::WiX::Toolchain;
 require File::List::Object;
 
-our $VERSION = '1.090_102';
+our $VERSION = '1.090_103';
 $VERSION = eval $VERSION; ## no critic (ProhibitStringyEval)
 
 Readonly my %CORE_MODULE_FIX => (
@@ -396,6 +396,10 @@ sub _skip_upgrade {
 	# already upgraded it.
 	return 1 if $module->id eq 'Term::ReadKey';
 
+	# DON'T try to install Win32API::Registry, we
+	# already upgraded it as far as we can.
+	return 1 if $module->id eq 'Win32API::Registry';
+
 	# If the ID is CGI::Carp, there's a bug in the index.
 	return 1 if $module->id eq 'CGI::Carp';
 
@@ -550,7 +554,7 @@ sub install_perl_bin {
 	my $self = shift;
 	my $perl = Perl::Dist::WiX::Asset::Perl->new(
 		parent => $self,
-		force  => $self->force,
+		force  => $self->forceperl() || $self->force(),
 		@_,
 	);
 
@@ -723,6 +727,11 @@ sub install_perl_toolchain {
 			# sites.
 			$force = 1;
 		}
+		if ( $dist =~ /Time-HiRes/msx ) {
+
+		   # Tests are so timing-sensitive they fail on their own sometimes.
+			$force = 1;
+		}
 		if ( $dist =~ /Term-ReadLine-Perl/msx ) {
 
 			# Does evil things when testing, and
@@ -744,6 +753,16 @@ sub install_perl_toolchain {
 			# 2.20 and 2.2002 are buggy on 5.8.9.
 			$dist = 'DAGOLDEN/ExtUtils-ParseXS-2.20_05.tar.gz';
 		}
+		if ( $dist =~ /Win32API-Registry-0 [.] 31/msx ) {
+
+			# 0.31 does not include a Makefile.PL.
+			$dist = 'BLM/Win32API-Registry-0.30.tar.gz';
+		}
+		if ( $dist =~ /Module-Build-/msx ) {
+
+			# 0.31 does not include a Makefile.PL.
+			$dist = 'DAGOLDEN/Module-Build-0.3500_01.tar.gz';
+		}
 		if ( $dist =~ /ExtUtils-MakeMaker-/msx ) {
 
 			# There are modules that overwrite portions of this one.
